from flask import Blueprint, request, jsonify
from geruecht.finanzer import LOGGER
from datetime import datetime
from geruecht.controller import accesTokenController, userController
from geruecht.model import MONEY

finanzer = Blueprint("finanzer", __name__)


@finanzer.route("/getFinanzerMain")
def _getFinanzer():
    """ Function for /getFinanzerMain

        Retrieves all User for the groupe 'moneymaster'

        Returns:
            A JSON-File with Users
            or ERROR 401 Permission Denied.
    """
    LOGGER.info("Get main for Finanzer")
    token = request.headers.get("Token")
    LOGGER.debug("Verify AccessToken with Token {}".format(token))
    accToken = accesTokenController.validateAccessToken(token, MONEY)
    if accToken:
        LOGGER.debug("Get all Useres")
        users = userController.getAllUsersfromDB()
        dic = {}
        for user in users:
            LOGGER.debug("Add User {} to ReturnValue".format(user))
            dic[user.uid] = user.toJSON()
            dic[user.uid]['creditList'] = {credit.year: credit.toJSON() for credit in user.geruechte}
            LOGGER.debug("ReturnValue is {}".format(dic))
        LOGGER.info("Send main for Finanzer")
        return jsonify(dic)
    LOGGER.info("Permission Denied")
    return jsonify({"error": "permission denied"}), 401

@finanzer.route("/finanzerAddAmount", methods=['POST'])
def _addAmount():
    """ Add Amount to User

        This Function add an amount to the user with posted userID.
        If year is not posted the default is the actual Year.
        If month is not posted the default is the actual Month.

        Returns:
            JSON-File with geruecht of year
            or ERROR 401 Permission Denied
    """
    LOGGER.info("Add Amount")
    token = request.headers.get("Token")
    LOGGER.debug("Verify AccessToken with Token {}".format(token))
    accToken = accesTokenController.validateAccessToken(token, MONEY)

    if accToken:
        data = request.get_json()
        LOGGER.debug("Get data {}".format(data))
        userID = data['userId']
        amount = int(data['amount'])
        LOGGER.debug("UserID is {} and amount is {}".format(userID, amount))
        try:
            year = int(data['year'])
        except KeyError as er:
            LOGGER.error("KeyError in year. Year is set to default.")
            year = datetime.now().year
        try:
            month = int(data['month'])
        except KeyError as er:
            LOGGER.error("KeyError in month. Month is set to default.")
            month = datetime.now().month
        LOGGER.debug("Year is {} and Month is {}".format(year, month))
        userController.addAmount(userID, amount, year=year, month=month)
        retVal = {geruecht.year: geruecht.toJSON() for geruecht in userController.getUser(userID).geruechte}
        LOGGER.info("Send updated Geruecht")
        return jsonify(retVal)
    LOGGER.info("Permission Denied")
    return jsonify({"error": "permission denied"}), 401

@finanzer.route("/finanzerAddCredit", methods=['POST'])
def _addCredit():
    """ Add Credit to User

        This Function add an credit to the user with posted userID.
        If year is not posted the default is the actual Year.
        If month is not posted the default is the actual Month.

        Returns:
            JSON-File with geruecht of year
            or ERROR 401 Permission Denied
    """
    LOGGER.info("Add Amount")
    token = request.headers.get("Token")
    LOGGER.debug("Verify AccessToken with Token {}".format(token))
    accToken = accesTokenController.validateAccessToken(token, MONEY)

    if accToken:

        data = request.get_json()
        print(data)
        LOGGER.debug("Get data {}".format(data))
        userID = data['userId']
        credit = int(data['credit'])
        LOGGER.debug("UserID is {} and credit is {}".format(userID, credit))

        try:
            year = int(data['year'])
        except KeyError as er:
            LOGGER.error("KeyError in year. Year is set to default.")
            year = datetime.now().year
        try:
            month = int(data['month'])
        except KeyError as er:
            LOGGER.error("KeyError in month. Month is set to default.")
            month = datetime.now().month

        LOGGER.debug("Year is {} and Month is {}".format(year, month))
        retVal = userController.addCredit(userID, credit, year=year, month=month).toJSON()
        retVal = {geruecht.year: geruecht.toJSON() for geruecht in userController.getUser(userID).geruechte}
        LOGGER.info("Send updated Geruecht")
        return jsonify(retVal)
    LOGGER.info("Permission Denied")
    return jsonify({"error": "permission denied"}), 401
